#!/usr/bin/env node
import minimist from '../node_modules/minimist/index.mjs';
import * as path from 'path';
import { createCatalog } from './util/createCatalog.mjs';
import { createFile } from './util/createFile.mjs';
import { debugFunction } from './util/debugFunction.mjs';
import { deleteCatalog } from './util/deleteCatalog.mjs';
import { downloadConfig } from './util/downloadConfig.mjs';
import { isFolderExist } from './util/isFolderExist.mjs';
import { redFile } from './util/readFile.mjs';
import { readPackageVersion } from './util/readVersionPackage.mjs';
import { updateJson } from './util/updateJson.mjs';

const init = async (args) => {
    const argSnpCatalog = args.snpConfig || args._[0];
    const argTemplate = args.template || args._[1];
    const argProjectCatalog = args.project || args._[2];
    const argRemoteRepository = args.remoteRepository || args._[3];
    const isDebug = args.debug || false;
    debugFunction(isDebug, '=== Start SNP INIT ===');
    debugFunction(isDebug, { args });
    const version = await readPackageVersion('./package.json');
    const projectCatalog = argProjectCatalog ? path.join(argProjectCatalog) + path.sep : path.join('./');
    const snpCatalog = argSnpCatalog
        ? path.join(projectCatalog, argSnpCatalog) + path.sep
        : path.join(projectCatalog, '.snp') + path.sep;
    const template = argTemplate ? argTemplate : 'node';
    const temporaryFolder = path.join(snpCatalog, './temporary') + path.sep;
    const snpConfigFile = 'snp.config.json';
    const remoteRepository = argRemoteRepository
        ? argRemoteRepository
        : 'https://raw.githubusercontent.com/SebastianWesolowski/s-update-manager/main/template/';
    return await createCatalog(temporaryFolder).then(() => {
        return {
            isDebug,
            projectCatalog,
            remoteRepository,
            sUpdaterVersion: version,
            snpCatalog,
            snpConfigFile,
            template,
            temporaryFolder,
        };
    });
};
const createConfigFile = async (config) => {
    debugFunction(config.isDebug, { config });
    const { snpCatalog, template, sUpdaterVersion } = config;
    config.snpConfigFile = path.join(snpCatalog, 'snp.config.json');
    await isFolderExist({
        folderPath: snpCatalog,
        createFolder: true,
    });
    await createFile({
        filePath: config.snpConfigFile,
        content: JSON.stringify({ sUpdaterVersion, template }),
        isDebug: config.isDebug,
    });
    debugFunction(config.isDebug, { sUpdaterVersion, template }, 'created snp config file');
    return config;
};
const downloadRemoteConfig = async (config) => {
    const { fileMap, templateVersion } = await downloadConfig(config);
    debugFunction(config.isDebug, { fileMap, templateVersion }, 'download form remote repo');
    const organizeFileMap = (fileMap) => {
        const files = {};
        fileMap.forEach((file) => {
            const fileName = file.substring(0, file.lastIndexOf('-'));
            if (!files[fileName]) {
                files[fileName] = [];
            }
            files[fileName].push(`${config.snpCatalog}/${file}`);
        });
        return { fileMap, files };
    };
    return { ...config, templateVersion, fileMap: organizeFileMap(fileMap) };
};
const buildConfig = async (config) => {
    const { fileMap, ...newConfigContent } = { ...config };
    await updateJson({ filePath: config.snpConfigFile, newContent: { ...newConfigContent } });
    const buildFile = async ({ fileMap, }) => {
        const files = fileMap.files;
        const contentFile = Object.keys(files);
        for (const fileName of contentFile) {
            const defaultFile = files[fileName].find((element) => element.includes('-default.md')) || '';
            const contentFile = await redFile(defaultFile);
            await createFile({
                filePath: path.join(config.projectCatalog, fileName),
                content: contentFile,
                isDebug: config.isDebug,
            });
        }
    };
    await buildFile(config);
    return config;
};
const cleanUp = async (config) => {
    debugFunction(config.isDebug, { config }, 'final config');
    await deleteCatalog(config.temporaryFolder, config.isDebug);
    debugFunction(config.isDebug, '=== final SNP INIT ===');
};
let args = minimist(process.argv.slice(2));
if (process.env.SDEBUG === 'true') {
    args = {
        _: [],
        snpConfig: './.snp',
        template: 'node',
        project: './test/fakeProjectRootfolder',
        remoteRepository: 'https://raw.githubusercontent.com/SebastianWesolowski/testTemplate/main/template/',
        debug: true,
    };
    console.log({ args });
}
init(args)
    .then((config) => createConfigFile(config))
    .then((config) => downloadRemoteConfig(config))
    .then((config) => buildConfig(config))
    .then((config) => cleanUp(config));

export { buildConfig, cleanUp, createConfigFile, downloadRemoteConfig, init };
